package jq.parse;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import oursland.naming.UIDGenerator;
import oursland.text.PrintStreamColumnFormatter;
import jq.qde.QDEQuantitySpace;
import jq.qde.QualitativeConstraint;
import jq.qde.QualitativeDifferentialEquation;
import jq.qde.QualitativeTransition;
import jq.qde.QDEState;
import jq.qde.constraint.AddConstraint;
import jq.qde.constraint.ConstantConstraint;
import jq.qde.constraint.ConstraintCorrespondence;
import jq.qde.constraint.DerivConstraint;
import jq.qde.constraint.MonotonicConstraint;
import jq.qde.constraint.MultConstraint;
import jq.qde.space.QualitativeVariable;
import jq.qde.space.TimeQuantitySpace;
import jq.qde.space.TimeVariable;
import jq.qde.space.VariableQuantitySpace;
import jq.qde.space.value.MagnitudeLandmark;
import jq.qde.space.value.QualitativeDirection;
import jq.qde.space.value.QualitativeMagnitude;
import jq.qde.space.value.TimeValue;
import jq.qsim.QSimListener;
import jq.qsim.QSimListenerAdapter;
import jq.qsim.QSimUtil;
import jq.util.PrintUtil;

/**
 * @author oursland
 */
public class JQParseListenerSetup implements JQParseListener {
	private QDEQuantitySpace qspace = new QDEQuantitySpace(new VariableQuantitySpace[0]);
	private LinkedList constraintList = new LinkedList();
	
	private QualitativeDifferentialEquation qde = null;
	private QDEState initialState = null;
	private LinkedList initialStateList = new LinkedList();
	private QDEState modifiedState = null;
//	private SimulationListener l = new SimulationPrinter(System.out);
	private QSimListener l = new QSimListenerAdapter();
	private UIDGenerator gen = new UIDGenerator();
	private PrintStreamColumnFormatter stateFormatter = new PrintStreamColumnFormatter(System.out, new int[] {4, 16, 32});
	private final HashMap varMap = new HashMap();

	public void qspace(String varName, String[] lmNames) {
		VariableQuantitySpace vqs = new VariableQuantitySpace(getVariable(varName), lmNames);
		qspace = qspace.add(vqs);
//		qspace.printSpecification(System.out, "");
//		System.out.println();
	}
	public void constraint(String constraintName, String[] args) throws IOException {
		QualitativeConstraint constraint = getConstraint(constraintName, args);
		constraintList.addLast(constraint);
//		System.out.println("constraint " + constraintName);
	}
	public void correspondence(String[] lmNames) throws IOException {
		QualitativeConstraint constraint = (QualitativeConstraint) constraintList.removeLast();
		if (lmNames.length != constraint.getVariableCount())
			throw new IOException("Expected "+ constraint.getVariableCount() +" arguments for "+constraint.getName()+". Found " + lmNames.length);


		MagnitudeLandmark[] lms = new MagnitudeLandmark[lmNames.length];
		for (int i = 0; i < lms.length; i++) {
			QualitativeVariable var = constraint.getVariable(i);
			lms[i] = var.getLandmark(lmNames[i]);
			validateLandmark(lms[i], "corresponding value");
		}
		ConstraintCorrespondence corr = new ConstraintCorrespondence(lms);
		constraint = constraint.addCorrespondingValue(corr);
		constraintList.addLast(constraint);

//		System.out.print("correspondence ");
//		for (int i = 0; i < lmNames.length; i++) {
//			System.out.print(lmNames[i]);
//			System.out.print(" ");
//		}
//		System.out.println();
		
//		constraint.printSpecification(System.out, "\t");
//		System.out.println();
	}
	public void printname(String varName, String name, String prefix) {
		QualitativeVariable var = getVariable(varName);
		var.setFullName(name);
		var.setLandmarkPrefix(prefix);
//		System.out.println("printname " + varName + " " + name + " " + prefix);
	}
	public void startInitial() throws IOException {
		if( qde == null ) {
			QualitativeConstraint[] constraints = (QualitativeConstraint[]) constraintList.toArray(new QualitativeConstraint[constraintList.size()]);
			qde = new QualitativeDifferentialEquation("", qspace, new TimeQuantitySpace(new TimeVariable("t_")), constraints, new QualitativeTransition[0]);
		}
		TimeValue t0 = (TimeValue) qde.tspace.getQValue(0);
		initialState = new QDEState(t0, qde);
	}
	public void init(String varName, String magName, String lowName, String highName, String dirName) throws IOException {
		QualitativeVariable var = getVariable(varName);
		QualitativeDirection dir = QualitativeDirection.getDirection(dirName);

		if( magName == null ) {
			MagnitudeLandmark low = var.getLandmark(lowName);
			validateLandmark(low, "initial state");
			MagnitudeLandmark high = var.getLandmark(highName);
			validateLandmark(high, "initial state");
			int lowIndex = qde.qspace.getVariableSpace(var).getQSpaceIndex(low);
			int highIndex = qde.qspace.getVariableSpace(var).getQSpaceIndex(high);
			if( lowIndex >= highIndex ) {
				throw new IOException("(" + lowName + " "+ highName + ") is not a valid interval. Landmarks are not ordered.");
			}
			if( lowIndex+2 != highIndex ) {
				throw new IOException("(" + lowName + " "+ highName + ") is not a valid interval. Landmarks are not adjacent.");
			}
			QualitativeMagnitude mag = qde.qspace.getVariableSpace(var).getQValue(lowIndex+1);
			initialState.setVariableValue(var, mag, dir);			
//			System.out.println("init " + varName + " (" + lowName + " " + highName + ") " + dirName);
		} else {
			MagnitudeLandmark mag = var.getLandmark(magName);
			validateLandmark(mag, "initial state");
			initialState.setVariableValue(var, mag, dir);
//			System.out.println("init " + varName + " " + magName + " " + dirName);
		}
	}
	public void endInitial() throws IOException {
		initialStateList.clear();
		QSimUtil.doInitialState(qde, initialState, gen, initialStateList, l);
		System.out.println(initialStateList.size() + " completions from initial values.");
		for (int i = 0; i < initialStateList.size(); i++) {
			PrintUtil.printState(stateFormatter, (QDEState) initialStateList.get(i));
		}
	}
	public void startModified() throws IOException {
		if( qde == null ) {
			QualitativeConstraint[] constraints = (QualitativeConstraint[]) constraintList.toArray(new QualitativeConstraint[constraintList.size()]);
			qde = new QualitativeDifferentialEquation("", qspace, new TimeQuantitySpace(new TimeVariable("t_")), constraints, new QualitativeTransition[0]);
		}
		TimeValue t0 = (TimeValue) qde.tspace.getQValue(0);
		modifiedState = new QDEState(t0, qde);
	}
	public void modified(String varName, String magName, String lowName, String highName, String dirName) throws IOException {
		QualitativeVariable var = getVariable(varName);
		QualitativeDirection dir = QualitativeDirection.getDirection(dirName);

		if( magName == null ) {
			MagnitudeLandmark low = var.getLandmark(lowName);
			validateLandmark(low, "modified state");
			MagnitudeLandmark high = var.getLandmark(highName);
			validateLandmark(high, "modified state");
			int lowIndex = qde.qspace.getVariableSpace(var).getQSpaceIndex(low);
			int highIndex = qde.qspace.getVariableSpace(var).getQSpaceIndex(high);
			if( lowIndex >= highIndex ) {
				throw new IOException("(" + lowName + " "+ highName + ") is not a valid interval. Landmarks are not ordered.");
			}
			if( lowIndex+2 != highIndex ) {
				throw new IOException("(" + lowName + " "+ highName + ") is not a valid interval. Landmarks are not adjacent.");
			}
			QualitativeMagnitude mag = qde.qspace.getVariableSpace(var).getQValue(lowIndex+1);
			modifiedState.setVariableValue(var, mag, dir);			
//			System.out.println("modified " + varName + " (" + lowName + " " + highName + ") " + dirName);
		} else {
			MagnitudeLandmark mag = var.getLandmark(magName);
			validateLandmark(mag, "initial state");
			modifiedState.setVariableValue(var, mag, dir);
//			System.out.println("modified " + varName + " " + magName + " " + dirName);
		}
	}
	public void endModified() throws IOException {
		initialStateList.removeFirst();
		QSimUtil.doInitialState(qde, modifiedState, gen, initialStateList, l);
		System.out.println(initialStateList.size() + " completions from modified values.");
		for (int i = 0; i < initialStateList.size(); i++) {
			PrintUtil.printState(stateFormatter, (QDEState) initialStateList.get(i));
		}
	}
	public void independent(String varName) {
		QualitativeVariable var = getVariable(varName);
		var.setIndependent(true);
//		System.out.println("independent " + varName);
	}
	public void eof() {
		qde.printSpecification(System.out, "");
		System.out.println();
		System.out.println("Initial State:");
		PrintUtil.printState(stateFormatter, initialState);
		System.out.println();
		//		SimulationPrinter listener = new SimulationPrinter(System.out);

		ArrayList totalSetList = new ArrayList();
		final int maxStates = 80;
		ArrayList currentStateList = new ArrayList(initialStateList);
		int currentStates = currentStateList.size();
		while(!currentStateList.isEmpty()) {
			totalSetList.addAll(currentStateList);
			QDEState[] currentSetArray = new QDEState[currentStateList.size()];
			currentSetArray = (QDEState[]) currentStateList.toArray(currentSetArray);
			currentStateList.clear();
			for(int i = 0; i < currentSetArray.length; i++) {
				QSimUtil.doNextStates(currentSetArray[i], gen, currentStateList, l);
			}
			currentStates += currentStateList.size();
			if(currentStates > maxStates) {
				System.out.println("Processed the maximum number of states.");
				break;
			}
		}
		QDEState[] states = (QDEState[]) totalSetList.toArray(new QDEState[currentStateList.size()]);
		System.out.println("Simulation Results:");
		for(int i = 0; i < states.length; i++) {
			PrintUtil.printState(stateFormatter, states[i]);
		}
	}
	
	public QualitativeVariable getVariable(String name) {
		Object var = varMap.get(name);
		if( var == null ) {
			var = new QualitativeVariable(name);
			varMap.put(name, var);
		}
		return (QualitativeVariable) var;
	}
	public QualitativeConstraint getConstraint(String name, String[] args) throws IOException {
		QualitativeVariable[] vars = new QualitativeVariable[args.length];
		for (int i = 0; i < vars.length; i++) {
			vars[i] = getVariable(args[i]);
		} 
		
		QualitativeConstraint rc;
		if (name.equalsIgnoreCase("d//dt")) {
			if (vars.length != 2)
				throw new IOException("Expected 2 arguments for d//t. Found " + vars.length);
			rc = new DerivConstraint(vars[0], vars[1]);
		} else if (name.equalsIgnoreCase("M+")) {
			if (vars.length != 2)
				throw new IOException("Expected 2 arguments for M+. Found " + vars.length);
			rc = new MonotonicConstraint(true, vars[0], vars[1]);
		} else if (name.equalsIgnoreCase("M-")) {
			if (vars.length != 2)
				throw new IOException("Expected 2 arguments for M-. Found " + vars.length);
			rc = new MonotonicConstraint(false, vars[0], vars[1]);
		} else if (name.equalsIgnoreCase("add")) {
			if (vars.length != 3)
				throw new IOException("Expected 3 arguments for add. Found " + vars.length);
			rc = new AddConstraint(vars[0], vars[1], vars[2]);
		} else if (name.equalsIgnoreCase("mult")) {
			if (vars.length != 3)
				throw new IOException("Expected 3 arguments for add. Found " + vars.length);
			rc = new MultConstraint(vars[0], vars[1], vars[2]);
		} else if (name.equalsIgnoreCase("constant")) {
			if (vars.length != 1)
				throw new IOException("Expected 1 arguments for constant. Found " + vars.length);
			rc = new ConstantConstraint(vars[0], new ConstraintCorrespondence[0]);
		} else {
			throw new IOException("Found unknown constraint type \"" + name + "\".");
		}
		return rc;
	}
	private void validateLandmark(MagnitudeLandmark lm, String context) throws IOException {
		if( !lm.isUnknown() ) {
			QualitativeVariable var = lm.getVariable();
			if (!qspace.getVariableSpace(var).contains(lm)) {
				throw new IOException("Landmark \"" + lm.getName() + "\" in " + context + " is not in " + var.getName() + " quantity space.");
			}
		}
	}
}
