package jq.parse;

import java.awt.Point;
import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import oursland.collection.IntegerPipe;

/**
 * @author oursland
 */
public class JQParser {
	private static final int	ROOT					= 0;
	private static final int	ROOT2				= 1;
	private static final int	QSPACE				= 2;
	private static final int	QSPACE_VAR_NAME		= 3;
	private static final int	QSPACE_VAR_LMS		= 4;
	private static final int	QSPACE_VAR_LM		= 5;
	private static final int	INDEPENDENT			= 6;
	private static final int	CONSTRAINTS			= 7;
	private static final int	CONSTRAINT_SPEC		= 8;
	private static final int	CONSTRAINT_SPEC_NAME	= 9;
	private static final int	CONSTRAINT_SPEC_ARGS	= 10;
	private static final int	CONSTRAINT_CORR		= 11;
	private static final int	CONSTRAINT_CORR_LM	= 12;
	private static final int	PRINT_NAMES			= 13;
	private static final int	NAME_SPEC_VAR		= 14;
	private static final int	NAME_SPEC_DESC		= 15;
	private static final int	NAME_SPEC_PRE		= 16;
	private static final int	NAME_SPEC_X			= 17;
	private static final int	INIT					= 18;
	private static final int	INIT_VAR_NAME		= 19;
	private static final int	INIT_VAR_VALUE		= 20;
	private static final int	INIT_VAR_MAG			= 21;
	private static final int	INIT_VAR_MAG_LOW		= 22;
	private static final int	INIT_VAR_MAG_HIGH	= 23;
	private static final int	INIT_VAR_MAG_X		= 24;
	private static final int	INIT_VAR_DIR			= 25;
	private static final int	INIT_VAR_DIR_X		= 26;
	private static final int	INIT_VAR_X			= 27;
	private static final int	MODIFIED				= 28;
	private static final int	MODIFIED_VAR_NAME	= 29;
	private static final int	MODIFIED_VAR_VALUE	= 30;
	private static final int	MODIFIED_VAR_MAG		= 31;
	private static final int	MODIFIED_VAR_MAG_LOW	= 32;
	private static final int	MODIFIED_VAR_MAG_HIGH	= 33;
	private static final int	MODIFIED_VAR_MAG_X	= 34;
	private static final int	MODIFIED_VAR_DIR		= 35;
	private static final int	MODIFIED_VAR_DIR_X	= 36;
	private static final int	MODIFIED_VAR_X		= 37;
	
	private String				dir;
	private String				mag;
	private String				low;
	private String				high;
	private String				prefix;
	private String				var;
	private ArrayList			corr				= new ArrayList();
	private ArrayList			args				= new ArrayList();
	private String				constraint;
	private ArrayList			landmarks		= new ArrayList();
	private String				name;
	private final JQParseListener	l;
	private int					column;
	private int					line;
	private InputStream			in;
	private IntegerPipe			buf				= new IntegerPipe(10);
	private int					depth			= 0;
	private ArrayList			parenLocation	= new ArrayList();
	private int					currentContext	= ROOT;

	public static void main(String[] args) {
		String filename = args[0];
		try {
			BufferedInputStream in = new BufferedInputStream(new FileInputStream(filename));
			JQParser parser = new JQParser(in, new JQParseListenerSetup());
			parser.parse();
			in.close();
		} catch(FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch(IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public JQParser(InputStream in, JQParseListener l) {
		this.in = in;
		this.l = l;
	}

	private void parse() throws IOException {
		while(nextToken());
	}

	private boolean nextToken() throws IOException {
		boolean notEOF = true;
		eatWhitespace();
		switch(peekChar()) {
			case -1:		this.eof(); notEOF = false; break;
			case '(':	this.openParen(Character.toString(nextChar())); break;
			case ')':	this.closeParen(Character.toString(nextChar())); break;
			case '\"':	this.stringToken(nextString()); break;
			case ';':	this.commentToken(nextComment()); break;
			default:		this.identifierToken(nextIdentifier());
		}
		return notEOF;
	}

	private String nextComment() throws IOException {
		assert peekChar() == ';';
		StringBuffer rc = new StringBuffer();
		while(true) {
			int raw = peekChar();
			if(raw == -1 || raw == '\n' || raw == '\r') { return rc.toString(); }
			rc.append(nextChar());
		}
	}

	private void eatWhitespace() throws IOException {
		while(true) {
			int ch = peekChar();
			if(ch == -1) { return; }
			if(!Character.isWhitespace((char) ch)) { return; }
			nextChar();
		}
	}

	private String nextIdentifier() throws IOException {
		StringBuffer rc = new StringBuffer();
		while(true) {
			int raw = peekChar();
			if(!isIdentifierCharacter(raw)) { return rc.toString(); }
			rc.append(nextChar());
		}
	}

	private boolean isIdentifierCharacter(int raw) {
		if(raw == -1) { return false; }
		if(Character.isWhitespace((char) raw)) { return false; }
		if(raw == '(' || raw == ')') { return false; }
		if(raw == ';') { return false; }
		return true;
	}

	private String nextString() throws IOException {
		assert peekChar() == '\"';
		StringBuffer rc = new StringBuffer();
		rc.append(nextChar());
		while(true) {
			int raw = peekChar();
			if(raw == -1) {
				throwIOException("Unexpected EOF while reading string.");
			}
			char ch = nextChar();
			rc.append(ch);
			if(ch == '\"') { return rc.toString(); }
		}
	}

	private int peekChar() throws IOException {
		if(buf.available() == 0) {
			loadBuffer();
		}
		return buf.peek();
	}

	private boolean	inReturn	= false;

	private char nextChar() throws IOException {
		if(buf.available() == 0) {
			loadBuffer();
		}
		int raw = buf.get();
		if(raw == -1) {
			throwIOException("EOF");
		}
		char ch = (char) raw;
		switch(ch) {
			case '\r': { // CR, return
				line++;
				column = 0;
				inReturn = true;
			} break;
			case '\n': { // LF, newline
				if(!inReturn) {
					line++;
					column = 0;
				}
				inReturn = false;
			} break;
			default: {
				column++;
				inReturn = false;
			} break;
		}
		return ch;
	}

	private void loadBuffer() throws IOException {
		int ch = in.read();
		switch(ch) {
			case '\\': {
				flushBuffer();
				throwIOException("Escaped characters are not supported.");
			} break;
			default: {
				buf.put(ch);
			} break;
		}
	}

	private void flushBuffer() throws IOException {
		while(buf.available() > 0) {
			nextChar();
		}
	}

	private void throwIOException(String msg) throws IOException {
		throw new IOException("Line " + line + ": Column " + column + " :: " + msg);
	}

	private void setContext(int context) {
		this.currentContext = context;
	}

	private int getCurrentContext() {
		return currentContext;
	}

	public void openParen(String token) throws IOException {
		switch(getCurrentContext()) {
			case ROOT:					setContext(ROOT2); break;
			case ROOT2:					throwIOException("Found opening parenthesis. Expected identifier."); break;
			case QSPACE:					setContext(QSPACE_VAR_NAME); break;
			case QSPACE_VAR_NAME:			throwIOException("Found opening parenthesis. Expected variable."); break;
			case QSPACE_VAR_LMS:			setContext(QSPACE_VAR_LM); landmarks.clear(); break;
			case QSPACE_VAR_LM:			throwIOException("Found opening parenthesis. Expected landmark."); break;
			case INDEPENDENT:			throwIOException("Found opening parenthesis. Expected variable or closing parenthesis."); break;
			case CONSTRAINTS:			setContext(CONSTRAINT_SPEC); break;
			case CONSTRAINT_SPEC:			setContext(CONSTRAINT_SPEC_NAME); args.clear(); break;
			case CONSTRAINT_SPEC_NAME:	throwIOException("Found opening parenthesis. Expected constraint name."); break;
			case CONSTRAINT_SPEC_ARGS:	throwIOException("Found opening parenthesis. Expected variable."); break;
			case CONSTRAINT_CORR:			setContext(CONSTRAINT_CORR_LM); corr.clear(); break;
			case CONSTRAINT_CORR_LM:		throwIOException("Found opening parenthesis. Expected landmark or closing parenthesis."); break;
			case PRINT_NAMES:			setContext(NAME_SPEC_VAR); break;
			case NAME_SPEC_VAR:			throwIOException("Found opening parenthesis. Expected variable."); break;
			case NAME_SPEC_DESC:			throwIOException("Found opening parenthesis. Expected string."); break;
			case NAME_SPEC_PRE:			throwIOException("Found opening parenthesis. Expected identifier."); break;
			case NAME_SPEC_X:			throwIOException("Found opening parenthesis. Expected closing parenthesis."); break;
			case INIT:					setContext(INIT_VAR_NAME); break;
			case INIT_VAR_NAME:			throwIOException("Found opening parenthesis. Expected variable."); break;
			case INIT_VAR_VALUE:			setContext(INIT_VAR_MAG); low = null; high = null; mag = null; break;
			case INIT_VAR_MAG:			setContext(INIT_VAR_MAG_LOW); break;
			case INIT_VAR_MAG_LOW:		throwIOException("Found opening parenthesis. Expected landmark."); break;
			case INIT_VAR_MAG_HIGH:		throwIOException("Found opening parenthesis. Expected landmark."); break;
			case INIT_VAR_MAG_X:			throwIOException("Found opening parenthesis. Expected closing parenthesis."); break;
			case INIT_VAR_DIR:			throwIOException("Found opening parenthesis. Expected direction."); break;
			case INIT_VAR_DIR_X:			throwIOException("Found opening parenthesis. Expected closing parenthesis."); break;
			case INIT_VAR_X:				throwIOException("Found opening parenthesis. Expected closing parenthesis."); break;
			case MODIFIED:				setContext(MODIFIED_VAR_NAME); break;
			case MODIFIED_VAR_NAME:		throwIOException("Found opening parenthesis. Expected variable."); break;
			case MODIFIED_VAR_VALUE:		setContext(MODIFIED_VAR_MAG); low = null; high = null; mag = null; break;
			case MODIFIED_VAR_MAG:		setContext(MODIFIED_VAR_MAG_LOW); break;
			case MODIFIED_VAR_MAG_LOW:	throwIOException("Found opening parenthesis. Expected landmark."); break;
			case MODIFIED_VAR_MAG_HIGH:	throwIOException("Found opening parenthesis. Expected landmark."); break;
			case MODIFIED_VAR_MAG_X:		throwIOException("Found opening parenthesis. Expected closing parenthesis."); break;
			case MODIFIED_VAR_DIR:		throwIOException("Found opening parenthesis. Expected direction."); break;
			case MODIFIED_VAR_DIR_X:		throwIOException("Found opening parenthesis. Expected closing parenthesis."); break;
			case MODIFIED_VAR_X:			throwIOException("Found opening parenthesis. Expected closing parenthesis."); break;
			default:						throwIOException("Found unexpected opening parenthesis. (" + getCurrentContext() + ")"); 
		}
		depth++;
		parenLocation.add(new Point(column, line));
	}

	public void closeParen(String token) throws IOException {
		if(depth == 0) {
			throwIOException("Found unmatched closing parenthesis.");
		}
		switch(getCurrentContext()) {
			case ROOT:					throwIOException("Found closing parenthesis. Expected opening parenthesis."); break;
			case ROOT2:					throwIOException("Found closing parenthesis. Expected identifier."); break;
			case QSPACE:					setContext(ROOT); break;
			case QSPACE_VAR_NAME:			throwIOException("Found closing parenthesis. Expected variable."); break;
			case QSPACE_VAR_LMS:			setContext(QSPACE); break;
			case QSPACE_VAR_LM:			setContext(QSPACE_VAR_LMS); l.qspace(var, (String[]) landmarks.toArray(new String[landmarks.size()])); break;
			case INDEPENDENT:			setContext(ROOT); break;
			case CONSTRAINTS:			setContext(ROOT); break;
			case CONSTRAINT_SPEC:			throwIOException("Found closing parenthesis. Expected opening parenthesis."); break;
			case CONSTRAINT_SPEC_NAME:	throwIOException("Found closing parenthesis. Expected constraint name."); break;
			case CONSTRAINT_SPEC_ARGS:	setContext(CONSTRAINT_CORR); l.constraint(constraint, (String[]) args.toArray(new String[args.size()])); break;
			case CONSTRAINT_CORR:			setContext(CONSTRAINTS); break;
			case CONSTRAINT_CORR_LM:		setContext(CONSTRAINT_CORR); l.correspondence((String[]) corr.toArray(new String[corr.size()])); break;
			case PRINT_NAMES:			setContext(ROOT); break;
			case NAME_SPEC_VAR:			throwIOException("Found closing parenthesis. Expected variable."); break;
			case NAME_SPEC_DESC:			throwIOException("Found closing parenthesis. Expected string."); break;
			case NAME_SPEC_PRE:			throwIOException("Found closing parenthesis. Expected identifier."); break;
			case NAME_SPEC_X:			setContext(PRINT_NAMES); l.printname(var, name, prefix); break;
			case INIT:					setContext(ROOT); l.endInitial(); break;
			case INIT_VAR_NAME:			throwIOException("Found closing parenthesis. Expected variable."); break;
			case INIT_VAR_VALUE:			setContext(INIT_VAR_MAG); break;
			case INIT_VAR_MAG:			throwIOException("Found closing parenthesis. Expected landmark."); break;
			case INIT_VAR_MAG_LOW:		throwIOException("Found closing parenthesis. Expected landmark."); break;
			case INIT_VAR_MAG_HIGH:		throwIOException("Found closing parenthesis. Expected landmark."); break;
			case INIT_VAR_MAG_X:			setContext(INIT_VAR_DIR); break;
			case INIT_VAR_DIR:			throwIOException("Found closing parenthesis. Expected direction."); break;
			case INIT_VAR_DIR_X:			setContext(INIT_VAR_X); l.init(var, mag, low, high, dir); break;
			case INIT_VAR_X:				setContext(INIT); break;
			case MODIFIED:				setContext(ROOT); l.endModified(); break;
			case MODIFIED_VAR_NAME:		throwIOException("Found closing parenthesis. Expected variable."); break;
			case MODIFIED_VAR_VALUE:		setContext(MODIFIED_VAR_MAG); break;
			case MODIFIED_VAR_MAG:		throwIOException("Found closing parenthesis. Expected landmark."); break;
			case MODIFIED_VAR_MAG_LOW:	throwIOException("Found closing parenthesis. Expected landmark."); break;
			case MODIFIED_VAR_MAG_HIGH:	throwIOException("Found closing parenthesis. Expected landmark."); break;
			case MODIFIED_VAR_MAG_X:		setContext(MODIFIED_VAR_DIR); break;
			case MODIFIED_VAR_DIR:		throwIOException("Found closing parenthesis. Expected direction."); break;
			case MODIFIED_VAR_DIR_X:		setContext(MODIFIED_VAR_X); l.modified(var, mag, low, high, dir); break;
			case MODIFIED_VAR_X:			setContext(MODIFIED); break;
			default:						throwIOException("Found unexpected closing parenthesis. (" + getCurrentContext() + ")"); break;
		}
		depth--;
		parenLocation.remove(parenLocation.size() - 1);
	}

	public void stringToken(String token) throws IOException {
		switch(getCurrentContext()) {
			case ROOT:					throwIOException("Found string. Expected opening parenthesis."); break;
			case ROOT2:					throwIOException("Found string. Expected identifier."); break;
			case QSPACE:					throwIOException("Found string. Expected opening parenthesis."); break;
			case QSPACE_VAR_NAME:			throwIOException("Found string. Expected variable."); break;
			case QSPACE_VAR_LMS:			throwIOException("Found string. Expected closing parenthesis."); break;
			case QSPACE_VAR_LM:			throwIOException("Found string. Expected landmark."); break;
			case INDEPENDENT:			throwIOException("Found string. Expected variable or closing parenthesis."); break;
			case CONSTRAINTS:			throwIOException("Found string. Expected opening parenthesis."); break;
			case CONSTRAINT_SPEC:			throwIOException("Found string. Expected opening parenthesis."); break;
			case CONSTRAINT_SPEC_NAME:	throwIOException("Found string. Expected constraint name."); break;
			case CONSTRAINT_SPEC_ARGS:	throwIOException("Found string. Expected variable."); break;
			case CONSTRAINT_CORR:			throwIOException("Found string. Expected opening parenthesis."); break;
			case CONSTRAINT_CORR_LM:		throwIOException("Found string. Expected landmark."); break;
			case PRINT_NAMES:			throwIOException("Found string. Expected opening parenthesis."); break;
			case NAME_SPEC_VAR:			throwIOException("Found string. Expected variable."); break;
			case NAME_SPEC_DESC:			setContext(NAME_SPEC_PRE); name = token; break;
			case NAME_SPEC_PRE:			throwIOException("Found string. Expected identifier."); break;
			case NAME_SPEC_X:			throwIOException("Found string. Expected closing parenthesis."); break;
			case INIT:					throwIOException("Found string. Expected opening parenthesis."); break;
			case INIT_VAR_NAME:			throwIOException("Found string. Expected variable."); break;
			case INIT_VAR_VALUE:			throwIOException("Found string. Expected opening parenthesis."); break;
			case INIT_VAR_MAG:			throwIOException("Found string. Expected landmark."); break;
			case INIT_VAR_MAG_LOW:		throwIOException("Found string. Expected landmark."); break;
			case INIT_VAR_MAG_HIGH:		throwIOException("Found string. Expected landmark."); break;
			case INIT_VAR_MAG_X:			throwIOException("Found string. Expected closing parenthesis."); break;
			case INIT_VAR_DIR:			throwIOException("Found string. Expected direction."); break;
			case INIT_VAR_DIR_X:			throwIOException("Found string. Expected direction."); break;
			case INIT_VAR_X:				throwIOException("Found string. Expected direction."); break;
			case MODIFIED:				throwIOException("Found string. Expected opening parenthesis."); break;
			case MODIFIED_VAR_NAME:		throwIOException("Found string. Expected variable."); break;
			case MODIFIED_VAR_VALUE:		throwIOException("Found string. Expected opening parenthesis."); break;
			case MODIFIED_VAR_MAG:		throwIOException("Found string. Expected landmark."); break;
			case MODIFIED_VAR_MAG_LOW:	throwIOException("Found string. Expected landmark."); break;
			case MODIFIED_VAR_MAG_HIGH:	throwIOException("Found string. Expected landmark."); break;
			case MODIFIED_VAR_MAG_X:		throwIOException("Found string. Expected closing parenthesis."); break;
			case MODIFIED_VAR_DIR:		throwIOException("Found string. Expected direction."); break;
			case MODIFIED_VAR_DIR_X:		throwIOException("Found string. Expected direction."); break;
			case MODIFIED_VAR_X:			throwIOException("Found string. Expected direction."); break;
			default:						throwIOException("Found unexpected string. (" + getCurrentContext() + ")");

		}
	}

	public void commentToken(String token) {}

	public void identifierToken(String token) throws IOException {
		switch(getCurrentContext()) {
			case ROOT:	throwIOException("Found identifier. Expected opening parenthesis."); break;
			case ROOT2: {
				if(token.equals("quantity-spaces")) {
				setContext(QSPACE);
			} else if(token.equals("independent")) {
				setContext(INDEPENDENT);
			} else if(token.equals("constraints")) {
				setContext(CONSTRAINTS);
			} else if(token.equals("print-names")) {
				setContext(PRINT_NAMES);
			} else if(token.equals("make-initial-state")) {
				setContext(INIT);
				l.startInitial();
			} else if(token.equals("make-modified-state")) {
				setContext(MODIFIED);
				l.startModified();
			} else {
				throwIOException("Found unknown section identifier \"" + token + "\".");
				}
			} break;
			case QSPACE:					throwIOException("Found identifier. Expected opening parenthesis."); break;
			case QSPACE_VAR_NAME:			setContext(QSPACE_VAR_LMS); var = token; break;
			case QSPACE_VAR_LMS:			throwIOException("Found identifier. Expected closing parenthesis."); break;
			case QSPACE_VAR_LM:			landmarks.add(token); break;
			case INDEPENDENT:			setContext(INDEPENDENT); l.independent(token); break;
			case CONSTRAINTS:			throwIOException("Found identifier. Expected opening parenthesis."); break;
			case CONSTRAINT_SPEC:			throwIOException("Found identifier. Expected opening parenthesis."); break;
			case CONSTRAINT_SPEC_NAME:	setContext(CONSTRAINT_SPEC_ARGS); constraint = token; break;
			case CONSTRAINT_SPEC_ARGS:	args.add(token); break;
			case CONSTRAINT_CORR:			throwIOException("Found identifier. Expected opening parenthesis."); break;
			case CONSTRAINT_CORR_LM:		corr.add(token); break;
			case PRINT_NAMES:			throwIOException("Found identifier. Expected opening parenthesis."); break;
			case NAME_SPEC_VAR:			setContext(NAME_SPEC_DESC); var = token; break;
			case NAME_SPEC_DESC:			throwIOException("Found identifier. Expected string."); break;
			case NAME_SPEC_PRE:			setContext(NAME_SPEC_X); prefix = token; break;
			case NAME_SPEC_X:			throwIOException("Found identifier. Expected closing parenthesis."); break;
			case INIT:					throwIOException("Found identifier. Expected opening parenthesis."); break;
			case INIT_VAR_NAME:			setContext(INIT_VAR_VALUE); var = token; break;
			case INIT_VAR_VALUE:			throwIOException("Found identifier. Expected opening parenthesis."); break;
			case INIT_VAR_MAG:			setContext(INIT_VAR_DIR); mag = token; break;
			case INIT_VAR_MAG_LOW:		setContext(INIT_VAR_MAG_HIGH); low = token; break;
			case INIT_VAR_MAG_HIGH:		setContext(INIT_VAR_MAG_X); high = token; break;
			case INIT_VAR_MAG_X:			throwIOException("Found identifier. Expected closing parenthesis."); break;
			case INIT_VAR_DIR:			setContext(INIT_VAR_DIR_X); dir = token; break;
			case INIT_VAR_DIR_X:			throwIOException("Found identifier. Expected closing parenthesis."); break;
			case INIT_VAR_X:				throwIOException("Found identifier. Expected closing parenthesis."); break;
			case MODIFIED:				throwIOException("Found identifier. Expected opening parenthesis."); break;
			case MODIFIED_VAR_NAME:		setContext(MODIFIED_VAR_VALUE); var = token; break;
			case MODIFIED_VAR_VALUE:		throwIOException("Found identifier. Expected opening parenthesis."); break;
			case MODIFIED_VAR_MAG:		setContext(MODIFIED_VAR_DIR); mag = token; break;
			case MODIFIED_VAR_MAG_LOW:	setContext(MODIFIED_VAR_MAG_HIGH); low = token; break;
			case MODIFIED_VAR_MAG_HIGH:	setContext(MODIFIED_VAR_MAG_X); high = token; break;
			case MODIFIED_VAR_MAG_X:		throwIOException("Found identifier. Expected closing parenthesis."); break;
			case MODIFIED_VAR_DIR:		setContext(MODIFIED_VAR_DIR_X); dir = token; break;
			case MODIFIED_VAR_DIR_X:		throwIOException("Found identifier. Expected closing parenthesis."); break;
			case MODIFIED_VAR_X:			throwIOException("Found identifier. Expected closing parenthesis.");
			default:						throwIOException("Found unexpected identifier. (" + getCurrentContext() + ")");
		}
	}

	public void eof() throws IOException {
		switch(getCurrentContext()) {
			case ROOT:					l.eof(); break;
			case ROOT2:					throwIOException("Found EOF. Expected identifier."); break;
			case QSPACE:					throwIOException("Found EOF. Expected opening parenthesis."); break;
			case QSPACE_VAR_NAME:			throwIOException("Found EOF. Expected variable."); break;
			case QSPACE_VAR_LMS:			throwIOException("Found EOF. Expected closing parenthesis."); break;
			case QSPACE_VAR_LM:			throwIOException("Found EOF. Expected landmark."); break;
			case INDEPENDENT:			throwIOException("Found EOF. Expected variable or closing parenthesis."); break;
			case CONSTRAINTS:			throwIOException("Found EOF. Expected opening parenthesis."); break;
			case CONSTRAINT_SPEC:			throwIOException("Found EOF. Expected opening parenthesis."); break;
			case CONSTRAINT_SPEC_NAME:	throwIOException("Found EOF. Expected constraint name."); break;
			case CONSTRAINT_SPEC_ARGS:	throwIOException("Found EOF. Expected variable."); break;
			case CONSTRAINT_CORR:			throwIOException("Found EOF. Expected opening parenthesis."); break;
			case CONSTRAINT_CORR_LM:		throwIOException("Found EOF. Expected landmark."); break;
			case PRINT_NAMES:			throwIOException("Found EOF. Expected opening parenthesis."); break;
			case NAME_SPEC_VAR:			throwIOException("Found EOF. Expected variable."); break;
			case NAME_SPEC_DESC:			throwIOException("Found EOF. Expected string."); break;
			case NAME_SPEC_PRE:			throwIOException("Found EOF. Expected identifier."); break;
			case NAME_SPEC_X:			throwIOException("Found EOF. Expected closing parenthesis."); break;
			case INIT:					throwIOException("Found EOF. Expected opening parenthesis."); break;
			case INIT_VAR_NAME:			throwIOException("Found EOF. Expected variable."); break;
			case INIT_VAR_VALUE:			throwIOException("Found EOF. Expected opening parenthesis."); break;
			case INIT_VAR_MAG:			throwIOException("Found EOF. Expected magnitude."); break;
			case INIT_VAR_MAG_LOW:		throwIOException("Found EOF. Expected landmark."); break;
			case INIT_VAR_MAG_HIGH:		throwIOException("Found EOF. Expected landmark."); break;
			case INIT_VAR_MAG_X:			throwIOException("Found EOF. Expected closing parenthesis."); break;
			case INIT_VAR_DIR:			throwIOException("Found EOF. Expected direction."); break;
			case INIT_VAR_DIR_X:			throwIOException("Found EOF. Expected closing parenthesis."); break;
			case INIT_VAR_X:				throwIOException("Found EOF. Expected closing parenthesis."); break;
			case MODIFIED:				throwIOException("Found EOF. Expected opening parenthesis."); break;
			case MODIFIED_VAR_NAME:		throwIOException("Found EOF. Expected variable."); break;
			case MODIFIED_VAR_VALUE:		throwIOException("Found EOF. Expected opening parenthesis."); break;
			case MODIFIED_VAR_MAG:		throwIOException("Found EOF. Expected magnitude."); break;
			case MODIFIED_VAR_MAG_LOW:	throwIOException("Found EOF. Expected landmark."); break;
			case MODIFIED_VAR_MAG_HIGH:	throwIOException("Found EOF. Expected landmark."); break;
			case MODIFIED_VAR_MAG_X:		throwIOException("Found EOF. Expected closing parenthesis."); break;
			case MODIFIED_VAR_DIR:		throwIOException("Found EOF. Expected direction."); break;
			case MODIFIED_VAR_DIR_X:		throwIOException("Found EOF. Expected closing parenthesis."); break;
			case MODIFIED_VAR_X:			throwIOException("Found EOF. Expected closing parenthesis."); break;
			default:						throwIOException("Found unexpected EOF. (" + getCurrentContext() + ")");
		}
	}
}