package jq.qde.constraint;

import jq.qde.QDEState;
import jq.qde.QualitativeConstraint;
import jq.qde.space.QualitativeVariable;
import jq.qde.space.value.QualitativeDirection;
import jq.qde.space.value.QualitativeValue;
import jq.qsim.QSimListener;

/**
 * @author oursland
 */
public class ConstantConstraint extends QualitativeConstraint {

	public ConstantConstraint(QualitativeVariable x, ConstraintCorrespondence[] corr) {
		super("constant", new QualitativeVariable[] {x}, corr);
	}

	public ConstantConstraint(QualitativeVariable x) {
		this(x, EMPTYCORR);
	}
	
	public boolean propagateConstraint(QDEState state, MagnitudeDelta change) {
		final QualitativeVariable var = change.getVariable();
		final QualitativeVariable x = this.getVariable(0);
		if( change.isDirectionChange() &&  x == var ) {
			return initConstraint(state);
		}
		return true;
	}

	public boolean initConstraint(QDEState state) {
		QualitativeDirection dir = state.getVariableDirection(getVariable(0));
		if( dir == QualitativeDirection.DEC ) {
			return false;
		} else if( dir == QualitativeDirection.INC ) {	
			return false;
		} else if( dir == QualitativeDirection.STD ) {	
			return true;
		} else if( dir == QualitativeDirection.UNKNOWN ) {
			state.setVariableDirection(getVariable(0), QualitativeDirection.STD);	
			return true;
		}
		return true;
	}

	public ConstraintValidationResult validateValues(QualitativeValue[] stateArray) {
		boolean rc = stateArray[0].direction == QualitativeDirection.ZERO;
		if( rc ) {
			return new ConstraintValidationResult(true, "OK", this, stateArray);
		} else {
			return new ConstraintValidationResult(false, "qdir mismatch", this, stateArray);
		}
	}

	public QualitativeConstraint createCopy(QualitativeVariable[] args, ConstraintCorrespondence[] corr) {
		return new ConstantConstraint(args[0], corr);
	}
	
	public boolean shouldCreateNewCorrespondence(QDEState state, QSimListener l) {
		return false;
	}

}
