package jq.qde.constraint;

import java.io.PrintStream;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import jq.qde.QDEState;
import jq.qde.QualitativeConstraint;
import jq.qde.space.QualitativeVariable;
import jq.qde.space.value.MagnitudeLandmark;
import jq.qde.space.value.QualitativeMagnitude;
import jq.qde.space.value.QualitativeValue;

/**
 * @author oursland
 */
public final class ConstraintCorrespondence {
	private final MagnitudeLandmark[] values;
	private final HashMap map = new HashMap();
	
	public ConstraintCorrespondence(MagnitudeLandmark x, MagnitudeLandmark y) {
		this.values = new MagnitudeLandmark[] { x, y };
		map.put(x.getVariable(), x);
		map.put(y.getVariable(), y);
	}
	
	public ConstraintCorrespondence(MagnitudeLandmark x, MagnitudeLandmark y, MagnitudeLandmark z) {
		this.values = new MagnitudeLandmark[] { x, y, z };
		map.put(x.getVariable(), x);
		map.put(y.getVariable(), y);
		map.put(z.getVariable(), z);
	}
	
	public ConstraintCorrespondence(MagnitudeLandmark[] lms) {
		this.values = (MagnitudeLandmark[]) lms.clone();
		for (int i = 0; i < lms.length; i++) {
			map.put(lms[i].getVariable(), lms[i]);
		}
	}
	
	public ConstraintCorrespondence(QualitativeConstraint conp, QDEState state) {
		this.values = new MagnitudeLandmark[conp.getVariableCount()];
		for (int i = 0; i < this.values.length; i++) {
			QualitativeVariable var = conp.getVariable(i);
			this.values[i] = (MagnitudeLandmark) state.getValue(var).magnitude;
			map.put(var, this.values[i]);
		}
	}

	public ConstraintCorrespondence(ConstraintCorrespondence cv) {
		this.values = cv.values;
		map.putAll(map);
	}
	
	public void printSpecification(PrintStream out, String prefix) {
		out.print(prefix);
		out.print("(");
		for (int i = 0; i < values.length; i++) {
			if( i > 0 ) {
				out.print(" ");
			}			
			out.print(values[i].getName());
		}
		out.print(")");
	}

	public MagnitudeLandmark getValue(QualitativeVariable var) {
		MagnitudeLandmark rc = (MagnitudeLandmark) map.get(var);
		if( rc == null ) {
			throw new Error("Variable " + var.getName() + " does not exist in corresponding value.");
		}
		return rc;
	}
	
	public MagnitudeLandmark getValue(QualitativeValue value) {
		return getValue(value.variable);
	}

	public MagnitudeLandmark getValue(int index) {
		return values[index];
	}

//	public ValueLandmark getCorrespondingValue(int index) {
//		return values[index];
//	}
	
	public int getCorrespondingValueCount() {
		return map.size();
	}

	public boolean matchesState(QDEState state) {
		for (Iterator iter = map.entrySet().iterator(); iter.hasNext();) {
			Map.Entry entry = (Map.Entry) iter.next();
			final QualitativeVariable var = (QualitativeVariable) entry.getKey();
			final QualitativeMagnitude stateMag = state.getValue(var).magnitude;
			final QualitativeMagnitude corrMag = (QualitativeMagnitude) entry.getValue();
			if( stateMag != corrMag ) {
				return false;
			}
		}
		return true;
	}
	
	public boolean equals(Object o) {
		boolean rc = false;
		if( o instanceof ConstraintCorrespondence ) {
			ConstraintCorrespondence c = (ConstraintCorrespondence) o;
			if( c == this ) {
				rc = true;
			} else if( values.length == c.values.length ) {
				rc = true;
				for (int i = 0; i < values.length; i++) {
					if( values[i] != c.values[i] ) {
						rc = false;
						break;
					}
				}
			}
		}
		return rc;
	}

//	public ValueLandmark getCorrespondingValue(QualitativeVariable var, int index) {
//		for (int i = 0; i < values.length; i++) {
//			if( var == values[i].getVariable() ) {
//				return values[(i+index+1)%values.length];
//			}
//		}
//		throw new Error("Variable " + var.getName() + " does not exist in corresponding value.");
//	}
}
