package jq.qde.constraint;

import jq.qde.QDEState;
import jq.qde.QualitativeConstraint;
import jq.qde.space.QualitativeVariable;
import jq.qde.space.value.QualitativeDirection;
import jq.qde.space.value.QualitativeMagnitude;
import jq.qde.space.value.QualitativeValue;
import jq.qsim.QSimListener;

/**
 * x = dy/dt
 * @author oursland
 */
public class DerivConstraint extends QualitativeConstraint {
	private DerivConstraint(QualitativeVariable[] args, ConstraintCorrespondence[] corr) {
		super("d//dt", args, EMPTYCORR);
	}

	public DerivConstraint(QualitativeVariable x, QualitativeVariable y) {
		super("d//dt", new QualitativeVariable[] {x, y}, EMPTYCORR);
	}

	public boolean propagateConstraint(QDEState state, MagnitudeDelta change) {
		final QualitativeVariable var = change.getVariable();

		final QualitativeVariable x = getVariable(0);
		final QualitativeVariable y = getVariable(1);
//		final QualitativeMagnitude xVal = state.getVariableMagnitude(x);
		final QualitativeMagnitude yVal = state.getVariableMagnitude(y);
		final QualitativeDirection xDir = state.getVariableDirection(x);
//		final QualitativeDirection yDir = state.getVariableDirection(y);

		final QualitativeDirection STD = QualitativeDirection.STD;
		final QualitativeDirection INC = QualitativeDirection.INC;
		final QualitativeDirection DEC = QualitativeDirection.DEC;
		if (var == x && change.isDirectionChange()) {
			if (xDir == STD) {
				if (yVal.isUnknown()) {
					System.out.println("Propagating direction of " + x.getName() + " to value of " + y.getName() + ".");
					state.setVariableMagnitude(y, y.ZERO);
					return true;
				} else {
					return yVal.isZero();
				}
			} else if (xDir == INC) {
				System.out.println(
					"TODO: Propagating positive direction of " + x.getName() + " to value of " + y.getName() + ".");
				return !yVal.isZero() && state.qspace.isPositive(yVal);
			} else if (xDir == DEC) {
				System.out.println(
					"TODO: Propagating negative direction of " + x.getName() + " to value of " + y.getName() + ".");
				return yVal.isZero() && state.qspace.isNegative(yVal);
			}
		} else if (var == y && change.isValueChange()) {
			if (yVal.isZero()) {
				if (xDir == QualitativeDirection.UNKNOWN) {
					System.out.println(
						"Propagating zero value of " + y.getName() + " to direction of " + y.getName() + ".");
					state.setVariableDirection(x, STD);
					return true;
				} else {
					return xDir == STD;
				}
			} else if (state.qspace.isPositive(yVal)) {
				if (xDir == QualitativeDirection.UNKNOWN) {
					System.out.println(
						"Propagating positive value of " + y.getName() + " to direction of " + y.getName() + ".");
					state.setVariableDirection(x, INC);
					return true;
				} else {
					return xDir == INC;
				}
			} else if (state.qspace.isNegative(yVal)) {
				if (xDir == QualitativeDirection.UNKNOWN) {
					System.out.println(
						"Propagating negative value of " + y.getName() + " to direction of " + y.getName() + ".");
					state.setVariableDirection(x, DEC);
					return true;
				} else {
					return xDir == DEC;
				}
			}
		}
		return true;
	}

	public boolean initConstraint(QDEState state) {
		return true;
	}

//	public int createTuples(StateFilter filter, ArrayList list) {
//		final VariableState[] stateArray = { null, null };
//		final QualitativeVariable x = getArg(0);
//		final QualitativeVariable y = getArg(1);
//		int foundTupleCount = 0;
//		Iterator xIter = filter.stateIterator(x);
//		while (xIter.hasNext()) {
//			stateArray[0] = (VariableState) xIter.next();
//
//			// if the xState is bad due to another constraint, then ignore it.
//			if( !filter.isValid(stateArray[0]) ) {
//				continue;
//			}
//
//			Iterator yIter = filter.stateIterator(y);
//			while (yIter.hasNext()) {
//				stateArray[1] = (VariableState) yIter.next();
//
//				// if the second state is bad due to another constraint, then ignore it.
//				if( !filter.isValid(stateArray[1]) ) {
//					continue;
//				}
//			
//				// Param 1's qdir must agree with param 2's relation to zero.
//				// Note that this test relies on the fact that GT=INC, EQ=STD,
//				// and LT=DEC.
//				if ( validateStates(stateArray) ) {
//					/*  Form a new tuple  */
//					list.add(new ConstraintTuple(this, stateArray[0], stateArray[1]));
//					filter.setConstraintUse(stateArray[0], this);
//					filter.setConstraintUse(stateArray[1], this);
//					foundTupleCount++;
////					System.out.print("Adding:");
////					xState.printSpecification(System.out, "\t");
////					stateArray[1].printSpecification(System.out, "\t");
////					System.out.println();
//				} else {
////					System.out.print("Reject:");
////					xState.printSpecification(System.out, "\t");
////					stateArray[1].printSpecification(System.out, "\t");
////					System.out.println();
//				}
//			}
//		}
//		return foundTupleCount;
//	}

	public ConstraintValidationResult validateValues(QualitativeValue[] stateArray) {
		QualitativeDirection dir = stateArray[0].direction;
		QualitativeMagnitude mag = stateArray[1].magnitude;
		QualitativeDirection sign = stateArray[1].qspace.getSign(mag);
		
//		System.out.print("Checking deriv");
//		dir.printSpecification(System.out, "\t");
//		mag.printSpecification(System.out, "\t");
//		sign.printSpecification(System.out, "\t");
//		System.out.println("\t"+(dir == sign));
		
		boolean rc = dir == sign;
		if( rc ) {
			return new ConstraintValidationResult(true, "OK", this, stateArray);
		} else {
			return new ConstraintValidationResult(false, "qdir mismatch", this, stateArray);
		}
	}
	
	public boolean shouldCreateNewCorrespondence(QDEState state, QSimListener l) {
		return false;
	}

	public QualitativeConstraint createCopy(QualitativeVariable[] args, ConstraintCorrespondence[] corr) {
		return new DerivConstraint(args, corr);
	}
}
