package jq.qde.space.value;

import java.io.PrintStream;
import jq.qde.space.QualitativeVariable;
import jq.qde.space.VariableQuantitySpace;

/**
 * @author oursland
 */
public class MagnitudeInterval implements QualitativeMagnitude {
	public final MagnitudeLandmark low;
	public final MagnitudeLandmark high;
	public final boolean lowOpen;
	public final boolean highOpen;

	
	/**
	 * ValueIntervals should only be created through a Variable.getInterval call.
	 * @param low
	 * @param high
	 */
	public MagnitudeInterval(MagnitudeLandmark low, MagnitudeLandmark high) {
		this(low, true, high, true);
		assert low.getVariable() == high.getVariable();
	}

	MagnitudeInterval(MagnitudeLandmark low, boolean lowOpen, MagnitudeLandmark high, boolean highOpen) {
		assert low.getVariable() == high.getVariable();
		this.low = low;
		this.high = high;
		this.lowOpen = lowOpen;
		this.highOpen = highOpen;
	}

	public void printSpecification(PrintStream out, String prefix) {
		out.print(prefix+toSpecification());
	}

	public String toSpecification() {
		return "(" + low.toSpecification() + " " + high.toSpecification() + ")";
	}

	public String toAbbreviation() {
		return "(" + low.toAbbreviation() + " " + high.toAbbreviation() + ")";
	}

	public MagnitudeInterval nextInterval(QualitativeDirection direction) {
		return this;
	}

	public MagnitudeLandmark nextLandmark(QualitativeDirection direction) {
		if(direction.isIncreasing()) {
			return high;
		} else if(direction.isDecreasing()) {
			return low;
		}
		return null;
	}

	public MagnitudeLandmark getHigh() {
		return high;
	}

	public MagnitudeLandmark getLow() {
		return low;
	}

	public QualitativeMagnitude[] getInitialValues(VariableQuantitySpace qspace) {
		final int lowQIndex = (qspace.getQSpaceIndex(low)) + (lowOpen?1:0);
		final int highQIndex = (qspace.getQSpaceIndex(high)) + (highOpen?0:1);;
		final int count = highQIndex - lowQIndex; 
		QualitativeMagnitude[] rc = new QualitativeMagnitude[count];
		for( int i = 0; i < count; i++ ) {
			rc[i] = qspace.getQValue(lowQIndex+i);
		}
		return rc;
	}

	public boolean isInfinite() {
		return high.isInfinite() && low.isInfinite();
	}

	public boolean isInterval() {
		return true;
	}

	public boolean isPoint() {
		return false;
	}

	public boolean isUnknown() {
		return high.isUnknown() && low.isUnknown();
	}
	
	public boolean isZero() {
		return high.isZero() && low.isZero();
	}

	public boolean equals(Object obj) {
		boolean rc = false;
		if( obj instanceof MagnitudeInterval ) {
			MagnitudeInterval vi = (MagnitudeInterval) obj;
			rc = (vi.low==this.low) && (vi.high==this.high)&&(vi.lowOpen==this.lowOpen)&&(vi.highOpen==this.highOpen);
		}
		return rc;
	}

	public QualitativeVariable getVariable() {
		return low.getVariable();
	}

	public QualitativeDirection getInfiniteSign() {
		return QualitativeDirection.ZERO;
	}
	
	public int hashCode() {
		return 0xffff0000&(high.hashCode()<<16) + 0x0000ffff&(low.hashCode());
	}
}
