package jq.qde.space.value;

import java.awt.Shape;
import java.io.PrintStream;
import java.util.HashMap;
import oursland.naming.NamableObject;

import jq.qde.space.QualitativeVariable;

/**
 * @author oursland
 */
public abstract class QualitativeDirection extends NamableObject {
	public static final int INC_KEY = 0;
	public static final int STD_KEY = 1;
	public static final int DEC_KEY = 2;
	public static final int UNK_KEY = 3;
	public static final int INC_MASK = 0x01;
	public static final int STD_MASK = 0x02;
	public static final int DEC_MASK = 0x04;
	public static final int UNK_MASK = 0x08;
	public static final int ANY_MASK = INC_MASK | STD_MASK | DEC_MASK;
	public static final QualitativeDirection UNKNOWN = new UnknownSign();
	public static final QualitativeDirection INC = new IncSign();
	public static final QualitativeDirection STD = new StdSign();
	public static final QualitativeDirection DEC = new DecSign();
//	private static final QualitativeDirection[] POSSIBLEVALUES = new QualitativeDirection[] {INC, STD, DEC};
	public static final QualitativeDirection POSITIVE = INC;
	public static final QualitativeDirection NEGATIVE = DEC;
	public static final QualitativeDirection ZERO = STD;
	
	private static final QualitativeDirection[] UNK_INIT = new QualitativeDirection[] {INC, STD, DEC};
	private static final QualitativeDirection[] INC_INIT = new QualitativeDirection[] {INC};
	private static final QualitativeDirection[] STD_INIT = new QualitativeDirection[] {STD};
	private static final QualitativeDirection[] DEC_INIT = new QualitativeDirection[] {DEC};

	private static final HashMap nameMap = new HashMap();
	static {
		nameMap.put("unk", UNKNOWN);
		nameMap.put("unknown", UNKNOWN);
		nameMap.put("?", UNKNOWN);
		nameMap.put("nil", UNKNOWN);

		nameMap.put("inc", INC);
		nameMap.put("std", STD);
		nameMap.put("dec", DEC);
		nameMap.put("+", INC);
		nameMap.put("0", STD);
		nameMap.put("-", DEC);
		nameMap.put("pos", INC);
		nameMap.put("zero", STD);
		nameMap.put("neg", DEC);
	}
	public static QualitativeDirection getDirection(String dirName) {
		return (QualitativeDirection) nameMap.get(dirName);
	}

	public final int key;
	public final int mask;
	protected QualitativeDirection(String name, int key, int mask) {
		super(name);
		this.key = key;
		this.mask = mask;
	}

	public boolean isSteady() {
		return false;
	}
	public boolean isIncreasing() {
		return false;
	}
	public boolean isDecreasing() {
		return false;
	}
	public boolean isUnknown() {
		return false;
	}
	public final boolean isZero() {
		return isSteady();
	}

	public void printSpecification(PrintStream out, String prefix) {
		out.print(prefix + toAbbreviatedString());
	}

	public abstract QualitativeMagnitude toFundamental(QualitativeVariable var);
	public abstract String toCompactString();
	public abstract String toAbbreviatedString();
	public abstract String toDirectionString();
	public abstract Shape toPlotSymbol();
	public abstract QualitativeDirection reverse();
	public abstract QualitativeDirection[] getPossibleCurrentValues();

	private static class UnknownSign extends QualitativeDirection {
		public UnknownSign() { super("?", UNK_KEY, UNK_MASK);	}
		public QualitativeMagnitude toFundamental(QualitativeVariable var) { return var.getInterval(var.MINF, var.INF); }
		public String toCompactString() { return "[?]"; }
		public String toAbbreviatedString() { return "?"; }
		public String toDirectionString() { return "ign"; }
		public Shape toPlotSymbol() { return null; }
		public QualitativeDirection reverse() { return UNKNOWN; }
		public boolean isUnknown() { return true; }
		public QualitativeDirection[] getPossibleCurrentValues() { return UNK_INIT; } 
	}
	private static class IncSign extends QualitativeDirection {
		public IncSign() { super("+", INC_KEY, INC_MASK);	}
		public QualitativeMagnitude toFundamental(QualitativeVariable var) { return var.getInterval(var.ZERO, var.INF); }
		public String toCompactString() { return "[+]"; }
		public String toAbbreviatedString() { return "+"; }
		public String toDirectionString() { return "inc"; }
		public Shape toPlotSymbol() { return null; }
		public QualitativeDirection reverse() { return DEC; }
		public boolean isIncreasing() { return true; };
		public QualitativeDirection[] getPossibleCurrentValues() { return INC_INIT; } 
	}
	private static class StdSign extends QualitativeDirection {
		public StdSign() { super("0", STD_KEY, STD_MASK);	}
		public QualitativeMagnitude toFundamental(QualitativeVariable var) { return var.ZERO; }
		public String toCompactString() { return "[0]"; }
		public String toAbbreviatedString() { return "0"; }
		public String toDirectionString() { return "std"; }
		public Shape toPlotSymbol() { return null; }
		public QualitativeDirection reverse() { return STD; }
		public boolean isSteady() { return true; }
		public QualitativeDirection[] getPossibleCurrentValues() { return STD_INIT; } 
	}
	private static class DecSign extends QualitativeDirection {
		public DecSign() { super("-", DEC_KEY, DEC_MASK);	}
		public QualitativeMagnitude toFundamental(QualitativeVariable var) { return var.getInterval(var.MINF, var.ZERO); }
		public String toCompactString() { return "[-]"; }
		public String toAbbreviatedString() { return "-"; }
		public String toDirectionString() { return "dec"; }
		public Shape toPlotSymbol() { return null; }
		public QualitativeDirection reverse() { return INC; }
		public boolean isDecreasing() { return true; }
		public QualitativeDirection[] getPossibleCurrentValues() { return DEC_INIT; } 
	}
}


