package jq.qsim;

import java.io.PrintStream;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import jq.qde.space.QualitativeVariable;
import jq.qde.space.VariableQuantitySpace;
import jq.qde.space.value.QualitativeValue;

/**
 * @author oursland
 */
public class VariablePossibleValues {
	public final VariableQuantitySpace qspace;
	final private QualitativeVariable variable;
	final private HashSet valueSet = new HashSet();
	final private ArrayList valueList = new ArrayList();

	public VariablePossibleValues(VariableQuantitySpace qspace) {
		this.variable = qspace.variable;
		this.qspace = qspace;
	}

	public void add(QualitativeValue value) {
		assert value.variable == this.variable;
		assert value.qspace == this.qspace;
		if( valueSet.add(value) ) {
			valueList.add(value);
		}
		
	}
	
	public void removeState(QualitativeValue value) {
		assert value.variable == this.variable;
		assert value.qspace == this.qspace;
		if( valueSet.remove(value) ) {
			valueList.remove(value);
		}
	}

	public void printSpecification(PrintStream out, String prefix) {
		Iterator iter = valueList.iterator();
		out.print(prefix);
		out.print("(");
		while( iter.hasNext() ) {
			QualitativeValue value = (QualitativeValue) iter.next();
			assert value.variable == this.variable;
			assert value.qspace == this.qspace;
			System.out.print(value.toSpecification());
			if(iter.hasNext()) {
				out.print(" ");
			}
		}
		out.print(")");
	}

	public int getPossibleValueCount() {
		return valueList.size();
	}

	public QualitativeValue getPossibleValue(int index) {
		return (QualitativeValue) valueList.get(index);
	}

	public QualitativeVariable getVariable() {
		return variable;
	}

	public Iterator valueIterator() {
		return valueList.iterator();
	}
}
