package jq.samplemodels;

import jq.qde.QDEQuantitySpace;
import jq.qde.QualitativeConstraint;
import jq.qde.QualitativeDifferentialEquation;
import jq.qde.QualitativeTransition;
import jq.qde.constraint.DerivConstraint;
import jq.qde.space.QualitativeVariable;
import jq.qde.space.VariableQuantitySpace;

/**
 * @author oursland
 */
public class Ball extends QualitativeDifferentialEquation {
	private static final QualitativeVariable Y = new QualitativeVariable("y"); // "height(ball)"
	private static final QualitativeVariable V = new QualitativeVariable("v"); // "velocity(ball)"
	private static final QualitativeVariable A = new QualitativeVariable("a"); // "acceleration(gravity)"
	
	private static final String[] yqsa = {"minf", "0", "inf"};
	private static final String[] vqsa = {"minf", "0", "vv", "inf"};
	private static final String[] aqsa = {"minf", "aa", "0"};

	
	private static final VariableQuantitySpace ySpace = new VariableQuantitySpace(Y, yqsa);
	private static final VariableQuantitySpace vSpace = new VariableQuantitySpace(V, vqsa);
	private static final VariableQuantitySpace aSpace = new VariableQuantitySpace(A, aqsa);
	
	static {		
		A.setIndependent(true);
	}

	public Ball() {
		super("Ball", createQuantitySpace(), createTimeSpace(), createConstraints(), createTransitions());
	}
	
	private static final QDEQuantitySpace createQuantitySpace() {
		return new QDEQuantitySpace(new VariableQuantitySpace[] {ySpace, vSpace, aSpace});
	}
	
	private static final QualitativeConstraint[] createConstraints() {
		DerivConstraint c1 = new DerivConstraint(Y, V);
		DerivConstraint c2 = new DerivConstraint(V, A);
		return new QualitativeConstraint[] {c1, c2};
	}
	
	private static final QualitativeTransition[] createTransitions() {
		return new QualitativeTransition[] {};
	}
}
