package jq.samplemodels;

import jq.qde.QDEQuantitySpace;
import jq.qde.QualitativeConstraint;
import jq.qde.QualitativeDifferentialEquation;
import jq.qde.QualitativeTransition;
import jq.qde.constraint.AddConstraint;
import jq.qde.constraint.ConstantConstraint;
import jq.qde.constraint.ConstraintCorrespondence;
import jq.qde.constraint.DerivConstraint;
import jq.qde.constraint.MonotonicConstraint;
import jq.qde.space.QualitativeVariable;
import jq.qde.space.VariableQuantitySpace;
import jq.qde.space.value.MagnitudeLandmark;
import jq.qde.space.value.QualitativeDirection;
import jq.qde.space.value.QualitativeValue;

/**
 * Section 2.5. Problem 1. p33.
 * @author oursland
 */
public class Bathtub extends QualitativeDifferentialEquation {
//	private static final LandmarkValue MINF = LandmarkValue.MINF;
	private static final QualitativeVariable AMOUNT = new QualitativeVariable("amount");
	private static final QualitativeVariable OUTFLOW = new QualitativeVariable("outflow");
	private static final QualitativeVariable INFLOW = new QualitativeVariable("inflow");
	private static final QualitativeVariable NETFLOW = new QualitativeVariable("netflow");
	
	private static final String[] aqsa = {"0", "full", "inf"};
	private static final String[] oqsa = {"0", "inf"};
	private static final String[] iqsa = {"0", "if*", "inf"};
	private static final String[] nqsa = {"minf", "0", "inf"};
	
	private static final MagnitudeLandmark FULL = AMOUNT.getLandmark("full");
//	private static final ValueLandmark IF_ = INFLOW.getLandmark("if*");

	private static final VariableQuantitySpace AMOUNTSpace = new VariableQuantitySpace(AMOUNT, aqsa);
	private static final VariableQuantitySpace OUTFLOWSpace = new VariableQuantitySpace(OUTFLOW, oqsa);
	private static final VariableQuantitySpace INFLOWSpace = new VariableQuantitySpace(INFLOW, iqsa);
	private static final VariableQuantitySpace NETFLOWSpace = new VariableQuantitySpace(NETFLOW, nqsa);

	public Bathtub() {
		super("Minimal-Bathtub", createQuantitySpace(), createTimeSpace(), createConstraints(), createTransitions());
		INFLOW.setIndependent(true);
	}
	
	private static final QDEQuantitySpace createQuantitySpace() {
		return new QDEQuantitySpace(new VariableQuantitySpace[] {AMOUNTSpace, OUTFLOWSpace, INFLOWSpace, NETFLOWSpace});
	}
	
	private static final QualitativeConstraint[] createConstraints() {
		ConstraintCorrespondence origin = new ConstraintCorrespondence(AMOUNT.ZERO, OUTFLOW.ZERO);
		ConstraintCorrespondence ends = new ConstraintCorrespondence(AMOUNT.INF, OUTFLOW.INF);
		ConstraintCorrespondence[] corr = {origin, ends};
		MonotonicConstraint c1 = new MonotonicConstraint(true, AMOUNT, OUTFLOW, corr);
		AddConstraint c2 = new AddConstraint(NETFLOW, OUTFLOW, INFLOW);
		DerivConstraint c3 = new DerivConstraint(AMOUNT, NETFLOW);
		ConstantConstraint c4 = new ConstantConstraint(INFLOW);
		return new QualitativeConstraint[] {c1, c2, c3, c4};
	}
	
	private static final QualitativeTransition[] createTransitions() {
		QualitativeValue value = new QualitativeValue(AMOUNTSpace, FULL, QualitativeDirection.INC);
		return new QualitativeTransition[] {new QualitativeTransition(value, new QualitativeDifferentialEquation("overflow"))};
	}
}
