package jq.util;

import java.io.PrintStream;
import java.util.Iterator;
import java.util.ListIterator;
import oursland.text.PrintStreamColumnFormatter;

import jq.qde.QDEState;
import jq.qde.QualitativeConstraint;
import jq.qde.constraint.ConstraintCorrespondence;
import jq.qde.space.QualitativeVariable;
import jq.qde.space.VariableQuantitySpace;
import jq.qde.space.value.MagnitudeLandmark;
import jq.qde.space.value.QualitativeValue;
import jq.qsim.QDEPossibleValues;
import jq.qsim.QSimFilter;
import jq.qsim.ValueTuple;

/**
 * @author oursland
 */
public class PrintUtil {
	
	
	public static void printCreatedStates(PrintStreamColumnFormatter format, QDEState[] states) {
//		System.out.println("Created States");
		for (int k = 0; k < states.length; k++) {
			final QDEState state = states[k];
			printState(format, state);
		}
	}

	private static void printSuccessorIds(PrintStream out, QDEState state) {
		if( state.getSuccessorCount() > 0 ) {
			for (int i = 0; i < state.getSuccessorCount(); i++) {
				out.print(state.getSuccessor(i).getIdentifier());
				out.print(" ");
			}
		} else {
			out.print("none");
		}
	}

	private static void printPredessorIds(PrintStream out, QDEState state) {
		if( state.getPredecessorCount() > 0 ) {
			for (int i = 0; i < state.getPredecessorCount(); i++) {
				out.print(state.getPredecessor(i).getIdentifier());
				out.print(" ");
			}
		} else {
			out.print("none");
		}
	}

	public static void printFilterResults(QSimFilter filter) {
		System.out.println("Filter Results:");
		Iterator constraintIter = filter.constraintIterator();
		while (constraintIter.hasNext()) {
			QualitativeConstraint constraint = (QualitativeConstraint) constraintIter.next();
			Iterator tupleIter = filter.tupleIterator(constraint);
			while (tupleIter.hasNext()) {
				ValueTuple tuple = (ValueTuple) tupleIter.next();
				System.out.print("\t");
				System.out.print(tuple.toSpecification());
				filter.printTupleStatistics(System.out, tuple);
				System.out.println();
			}
		}
		System.out.println();
	}

	public static void printConstraintTuples(QSimFilter filter) {
		System.out.println("Constraint Tuples:");
		Iterator constraintIter = filter.constraintIterator();
		while (constraintIter.hasNext()) {
			QualitativeConstraint constraint = (QualitativeConstraint) constraintIter.next();
			Iterator tupleIter = filter.tupleIterator(constraint);
			while (tupleIter.hasNext()) {
				ValueTuple tuple = (ValueTuple) tupleIter.next();
				System.out.print("\t");
				System.out.print(tuple.toSpecification());
				System.out.println();
		
			}
		}
		System.out.println();
	}

	public static void printPossibleValues(QDEPossibleValues possibleStates) {
		for( int i = 0; i < possibleStates.getQuantitySpace().getVariableCount(); i++ ) {
			QualitativeVariable var = possibleStates.getQuantitySpace().getVariable(i);
			System.out.println(var.getName() + ":\t" + possibleStates.getPossibleValueCount(var) + " pvals:");
			Iterator iter2 = possibleStates.valueIterator(var);
			while (iter2.hasNext()) {
				QualitativeValue ps = (QualitativeValue) iter2.next();
				System.out.print("\t");
				System.out.println(ps.toSpecification());
			}
		}
		System.out.println();
	}

	public static void printSigns(PrintStream out, VariableQuantitySpace qspace) {
		ListIterator iter = qspace.landmarkListIterator();
		while (iter.hasNext()) {
			MagnitudeLandmark landmark = (MagnitudeLandmark) iter.next();
			out.print(qspace.getSign(landmark).toAbbreviatedString());
			out.print(" ");
			out.print(landmark.getName());
			out.println();
		}
	}

	public void printStates(PrintStream out, QDEState[] stateArray, QualitativeConstraint constraint) {
		out.print("Building Constraint ");
		constraint.printSpecification(out, "");
		out.println();
		for (int i = 0; i < stateArray.length; i++) {
			stateArray[i].print(out, "");
			System.out.println();
		}
	}

	public static void printState(PrintStreamColumnFormatter format, QDEState state) {
		format.out.println("State " + state.getIdentifier() + "     Time = " + state.getTime().toAbbreviation());
		
		format.nextColumn("Predecessor states: ");
		printPredessorIds(format.out, state);
		format.nextLine();
		
		format.nextColumn("Successor states:   ");
		printSuccessorIds(format.out, state);
		format.nextLine();
		
		format.nextColumn("Parameter");
		format.nextColumn("Value");
		format.nextColumn("Quantity Space");
		format.nextLine();
		for (int i = 0; i < state.getVariableCount(); i++) {
			VariableQuantitySpace variableSpace = state.qspace.getVariableSpace(i);
			QualitativeValue value = state.getValue(variableSpace.variable);
			format.nextColumn(value.getVariable().getName());
			format.nextColumn(value.toAbbreviation());
			format.nextColumn(variableSpace.toAbbreviation());
			format.nextLine();			
		}
	}

	public static void printCorrespondence(ConstraintCorrespondence corr) {
		// TODO Auto-generated method stub
		
	}

	public static void printCorrespondence(PrintStream out, ConstraintCorrespondence corr) {
		// TODO Auto-generated method stub - copy prcorr
		
	}

}
